function [F,H,p,rho,r1,r2,L]=hyper_em(X,T)

%initialize
[n,d]=size(X);
mu=-d/log2(exp(1)); %uniform distribution
%mu=1./2^d;

p=zeros(1,T);
rho=zeros(n,T);
r1=zeros(n,T);
L=zeros(1,T);

p(1)=.5;
r1(:,1)=.5*ones(n,1);
r2=r1;
rho(:,1)=ones(n,1);


for t=1:T
    fprintf('EM iteration: %d\n',t);
    %E-step
    [Fx,H]=hyperpmf_train(X,rho(:,t));
    F(:,t)=Fx;
    r1(:,t+1)= F(:,t) + log(1-p(t)+realmin);
    r2(:,t+1)= mu + log(p(t)+realmin);
    C=logsumexp([r1(:,t+1) r2(:,t+1)],2);
    r1(:,t+1)= exp(r1(:,t+1)-C); 
    r2(:,t+1)= exp(r2(:,t+1)-C);
    
    %M-step
    p(t+1)=sum(r2(:,t+1))./sum(r1(:,t+1)+r2(:,t+1));
    p=max(p,realmin); %bound away from 0
    rho(:,t+1)=n*(1-p(t+1))*r1(:,t+1)./sum((1-p(t+1))*r1(:,t+1)+p(t+1)*r2(:,t+1));

    %Expected log-likelihood
    L(t)=sum( r1(:,t).*(log(1-p(t))+F(:,t)) + r2(:,t).*(log(p(t))+mu) );
    
    %Test convergence
    if t>1 && abs(L(t)-L(t-1))<1e-20
        break
    end
end

F=F(:,1:t);
L=L(1:t);
p=p(1:t);
rho=rho(:,1:t);
r1=r1(:,1:t);
r2=r2(:,1:t);